/*!
 * @file    	APM32_MINI.c
 * @brief	 	This file provides set of firmware functions to manage Leds, push-button and COM ports
 *
 * @details		
 *
 * @version		V1.0.0
 * @author  	
 * @date    	2019-9-3
 *
 */
#include "APM32_MINI.h"

/**
 * @brief	Define IO Struct
 */
typedef struct
{
	GPIO_T *port;
	uint16_t pin;
	RCM_APB2_PERIPH_T clk;
}APM32_MINI_IO_T;

/**
 * @brief	Define LED IO
 */
const APM32_MINI_IO_T g_ledIO[] = 
{
	{APM32_MINI_LED2_GPIO_PORT, APM32_MINI_LED2_PIN, APM32_MINI_LED2_GPIO_CLK},
	{APM32_MINI_LED3_GPIO_PORT, APM32_MINI_LED3_PIN, APM32_MINI_LED3_GPIO_CLK},
};

/**
 * @brief	Define BUTTON IO
 */
const APM32_MINI_IO_T g_buttonIO[] = 
{
	{APM32_MINI_BUTTON1_GPIO_PORT, APM32_MINI_BUTTON1_PIN, APM32_MINI_BUTTON1_GPIO_CLK},
	{APM32_MINI_BUTTON2_GPIO_PORT, APM32_MINI_BUTTON2_PIN, APM32_MINI_BUTTON2_GPIO_CLK},
};

/**
 * @brief	Define COM TX IO
 */
const APM32_MINI_IO_T g_comTxIO[] = 
{
	{APM32_MINI_COM1_TX_GPIO_PORT, APM32_MINI_COM1_TX_PIN, APM32_MINI_COM1_TX_GPIO_CLK},
	{APM32_MINI_COM2_TX_GPIO_PORT, APM32_MINI_COM2_TX_PIN, APM32_MINI_COM2_TX_GPIO_CLK},
};

/**
 * @brief	Define COM RX IO
 */
const APM32_MINI_IO_T g_comRxIO[] = 
{
	{APM32_MINI_COM1_RX_GPIO_PORT, APM32_MINI_COM1_RX_PIN, APM32_MINI_COM1_RX_GPIO_CLK},
	{APM32_MINI_COM2_RX_GPIO_PORT, APM32_MINI_COM2_RX_PIN, APM32_MINI_COM2_RX_GPIO_CLK},
};

/*!
 * @brief		LED Init
 *
 * @param		led		Specifies the Led to be configured
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */	
void APM_MINI_LedInit(APM32_MINI_LED_T led)
{
	GPIO_ConfigStruct_T  GPIO_InitStructure;
	
	/** Enable GPIO clock */
	RCM_EnableAPB2PeriphClock(g_ledIO[led].clk);

	/** GPIO Config */
	GPIO_InitStructure.pin = g_ledIO[led].pin;
	GPIO_InitStructure.mode = GPIO_MODE_OUT_PP;
	GPIO_InitStructure.speed = GPIO_SPEED_50MHz;
	GPIO_Config(g_ledIO[led].port, &GPIO_InitStructure);
	
	/** Led off */
	GPIO_SetBits(g_ledIO[led].port, g_ledIO[led].pin);
}

/*!
 * @brief		Turns selected LED On
 *
 * @param		led		Specifies the Led to be set on
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */	
void APM_MINI_LedOn(APM32_MINI_LED_T led)
{
	GPIO_ResetBits(g_ledIO[led].port, g_ledIO[led].pin);
}

/*!
 * @brief		Turns selected LED Off
 *
 * @param		led		Specifies the Led to be set off
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */
void APM_MINI_LedOff(APM32_MINI_LED_T led)
{
	GPIO_SetBits(g_ledIO[led].port, g_ledIO[led].pin); 
}

/*!
 * @brief		Toggles the selected LED
 *
 * @param		led		Specifies the Led to be set toggled
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */
void APM_MINI_LedToggle(APM32_MINI_LED_T led)
{
	uint16_t portValue = 0;
	
	portValue = GPIO_ReadOutputPort(g_ledIO[led].port);
	portValue ^= g_ledIO[led].pin;
	GPIO_WriteOutputPort(g_ledIO[led].port, portValue);
}

/*!
 * @brief		Configures COM port
 *
 * @param		com					Specifies the COM port to be configured
 *
 * @param		USART_InitStruct	pointer to a USART_InitTypeDef structure that
 *									contains the configuration information for the specified USART peripheral
 *
 * @retval		None
 *
 * @see 
 *
 * @note 
 */
void APM_MINI_COMInit(APM32_MINI_COM_T com, USART_ConfigStruct_T* USART_InitStruct)
{
	USART_T *usart = NULL;
	GPIO_ConfigStruct_T GPIO_InitStructure;
	
	if (com == COM1)
	{
		RCM_EnableAPB2PeriphClock(APM32_MINI_COM1_CLK); 
		usart = APM32_MINI_COM1;
	}
	else
	{
		/* Enable the USART2 Pins Software Remapping */
		//GPIO_ConfigPinRemap(GPIO_REMAP_USART2);
		RCM_EnableAPB1PeriphClock(APM32_MINI_COM2_CLK);
		usart = APM32_MINI_COM2;
	}
	
	/* Enable TX Pin clock */
	RCM_EnableAPB2PeriphClock(g_comTxIO[com].clk);
	
	/* Configure USART Tx as alternate function push-pull */
	GPIO_InitStructure.mode = GPIO_MODE_AF_PP;
	GPIO_InitStructure.pin = g_comTxIO[com].pin;
	GPIO_InitStructure.speed = GPIO_SPEED_50MHz;
	GPIO_Config(g_comTxIO[com].port, &GPIO_InitStructure);

	/* Enable RX Pin clock */
	RCM_EnableAPB2PeriphClock(g_comRxIO[com].clk);
	
	/* Configure USART Rx as input floating */
	GPIO_InitStructure.mode = GPIO_MODE_IN_FLOATING;
	GPIO_InitStructure.pin = g_comRxIO[com].pin;
	GPIO_Config(g_comRxIO[com].port, &GPIO_InitStructure);

	/* USART configuration */
	USART_Config(usart, USART_InitStruct);

	/* Enable USART */
	USART_Enable(usart);
}
